"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateReader = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const config_1 = require("./_constantes/config");
const promises_1 = require("fs/promises");
const ErrorMsgs_1 = require("./_constantes/ErrorMsgs");
class TemplateReader {
    constructor(templateName, templatePath) {
        this.templateName = templateName;
        this.templatePath = templatePath;
        const matchResult = templatePath.match(/([^\/]*)\/*$/);
        this.templateDirName = matchResult[1];
    }
    static getTemplateConfigFile(templatePath) {
        const configFilePath = `${templatePath}/${config_1.nameConfigFile}`;
        try {
            if (!fs_1.default.existsSync(configFilePath)) {
                const matchResult = templatePath.match(/([^\/]*)\/*$/);
                return new Error(ErrorMsgs_1.ErrorMsgs.CONFIG_FIlE_MISS(matchResult[1]));
            }
            const fileBuffer = fs_1.default.readFileSync(path_1.default.resolve(configFilePath));
            return JSON.parse(fileBuffer.toString());
        }
        catch (err) {
            return new Error(ErrorMsgs_1.ErrorMsgs.UNEXPECTED_ERR("la récupération du fichier de configuration", err));
        }
    }
    static checkConfigFileProperties(configFile) {
        const errors = [];
        if (!configFile.template || typeof (configFile.template) !== "string")
            errors.push(ErrorMsgs_1.ErrorMsgs.CONFIG_FILE_INVALID('template', "string(min: 1 char)"));
        if (!configFile.componentWorkDir)
            errors.push(ErrorMsgs_1.ErrorMsgs.CONFIG_FILE_INVALID('componentWorkDir', "string(min: 1 char) | Object "));
        if (typeof configFile.componentWorkDir === "object") {
            if (!configFile.componentWorkDir.rootWorkDir || typeof configFile.componentWorkDir.rootWorkDir !== "string")
                errors.push(ErrorMsgs_1.ErrorMsgs.CONFIG_FILE_INVALID('componentWorkDir.rootWorkDir', "string(min: 1 char)"));
            else if (configFile.componentWorkDir.rootWorkDir.startsWith("/"))
                errors.push(ErrorMsgs_1.ErrorMsgs.ROOT_PATH_INVALID_VALUE(configFile.componentWorkDir.rootWorkDir));
            if (!configFile.componentWorkDir.hasOwnProperty("extensionWorkDir") || typeof configFile.componentWorkDir.extensionWorkDir !== "string")
                errors.push(ErrorMsgs_1.ErrorMsgs.CONFIG_FILE_INVALID('componentWorkDir.extensionWorkDir', "string(min: 0 char)"));
            else if (configFile.componentWorkDir.extensionWorkDir.startsWith("/"))
                errors.push(ErrorMsgs_1.ErrorMsgs.EXTENTION_PATH_INVALID_VALUE(configFile.componentWorkDir.extensionWorkDir));
        }
        else if (typeof configFile.componentWorkDir !== "string")
            errors.push(ErrorMsgs_1.ErrorMsgs.CONFIG_FILE_INVALID('componentWorkDir', "string(min: 1 char) | Object "));
        if (typeof configFile.componentWorkDir === "string" && configFile.componentWorkDir.startsWith("/"))
            errors.push(ErrorMsgs_1.ErrorMsgs.PROJECT_PATH_INVALID_VALUE(configFile.componentWorkDir));
        if (errors.length)
            return new Error(errors.join("\n"));
    }
    getTemplateFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield (0, promises_1.readdir)(this.templatePath, { withFileTypes: true });
                const files = result.filter(file => file.isFile() && file.name !== config_1.nameConfigFile);
                return files.map(file => {
                    const filePath = `${this.templatePath}/${file.name}`;
                    const fileBuffer = fs_1.default.readFileSync(path_1.default.resolve(filePath));
                    return {
                        fileName: file.name,
                        filePath,
                        fileData: fileBuffer.toString()
                    };
                });
            }
            catch (err) {
                return new Error(ErrorMsgs_1.ErrorMsgs.UNEXPECTED_ERR("la récupération des fichiers du template", err));
            }
        });
    }
}
exports.TemplateReader = TemplateReader;
//# sourceMappingURL=TemplateReader.js.map