"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateRepo = void 0;
const TemplateReader_1 = require("./TemplateReader");
const ErrorMsgs_1 = require("./_constantes/ErrorMsgs");
const conf_1 = __importDefault(require("conf"));
const Initializer_1 = require("./commands/Initializer");
const fs_1 = __importDefault(require("fs"));
const InfoMsgs_1 = require("./_constantes/InfoMsgs");
class TEMPLATE_NOT_FOUND extends Error {
    constructor(message) {
        super(message);
        this.name = "TEMPLATE_NOT_FOUND_ERROR";
    }
}
class TemplateRepository {
    constructor(Store, initialiser) {
        this.Store = Store;
        this.initialiser = initialiser;
    }
    clear() {
        this.Store.clear();
    }
    saveTemplate(templateName, templatePath) {
        this.Store.set(templateName, templatePath);
    }
    getOneMetadata(templateName) {
        return __awaiter(this, void 0, void 0, function* () {
            let metadataResult = this._getMetadata(templateName);
            if (metadataResult instanceof TEMPLATE_NOT_FOUND) {
                const reloadResult = yield this._reloadCLI(templateName);
                if (reloadResult instanceof Error)
                    return reloadResult;
                return reloadResult;
            }
            else if (metadataResult instanceof Error)
                return metadataResult;
            else {
                const consistencyResult = this._checkTemplateConsistency(templateName, metadataResult.configFile);
                if (consistencyResult instanceof Error)
                    return consistencyResult;
                else if (consistencyResult === true)
                    return metadataResult;
                const reloadResult = yield this._reloadCLI(templateName);
                if (reloadResult instanceof Error)
                    return reloadResult;
                return reloadResult;
            }
        });
    }
    getAllMetadata() {
        return __awaiter(this, void 0, void 0, function* () {
            const reloadResult = yield this.initialiser.execute();
            if (reloadResult instanceof Error)
                return reloadResult;
            const entriesResult = Object.entries(this.Store.store);
            let metadatas = [];
            for (const entry of entriesResult) {
                const templateName = entry[0];
                const templateDirPath = entry[1];
                const configFileResult = TemplateReader_1.TemplateReader.getTemplateConfigFile(templateDirPath);
                if (configFileResult instanceof Error)
                    return configFileResult;
                metadatas.push({
                    path: templateDirPath,
                    configFile: configFileResult
                });
            }
            return metadatas;
        });
    }
    _reloadCLI(templateName) {
        return __awaiter(this, void 0, void 0, function* () {
            const reloadResult = yield this.initialiser.execute();
            if (reloadResult instanceof Error)
                return reloadResult;
            const metadataResult = this._getMetadata(templateName);
            if (metadataResult instanceof Error)
                return metadataResult;
            const consistencyResult = this._checkTemplateConsistency(templateName, metadataResult.configFile);
            if (consistencyResult instanceof Error)
                return consistencyResult;
            else if (consistencyResult === false)
                return new Error(ErrorMsgs_1.ErrorMsgs.CACHE_UNEXPECTED_INCONSISTENCY);
            return metadataResult;
        });
    }
    _checkTemplateConsistency(cacheTempName, configFile) {
        const checkingResult = TemplateReader_1.TemplateReader.checkConfigFileProperties(configFile);
        if (checkingResult instanceof Error)
            return checkingResult;
        return configFile.template === cacheTempName;
    }
    _getMetadata(templateName) {
        const path = this.Store.get(templateName);
        if (!path || !fs_1.default.existsSync(path))
            return new TEMPLATE_NOT_FOUND(ErrorMsgs_1.ErrorMsgs.TEMPLATE_NOT_EXIST(templateName));
        const configFileResult = TemplateReader_1.TemplateReader.getTemplateConfigFile(path);
        if (configFileResult instanceof Error)
            return configFileResult;
        return { path, configFile: configFileResult };
    }
}
exports.TemplateRepo = new TemplateRepository(new conf_1.default(), new Initializer_1.Initializer(InfoMsgs_1.InfoMsgs.SUCCESS_RELOAD, false));
//# sourceMappingURL=TemplateRepository.js.map