"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Initializer = void 0;
const fs_1 = __importDefault(require("fs"));
const config_1 = require("../_constantes/config");
const path_1 = __importDefault(require("path"));
const promises_1 = require("fs/promises");
const TemplateReader_1 = require("../TemplateReader");
const TemplateRepository_1 = require("../TemplateRepository");
const FileHandler_1 = require("../FileHandler");
const ErrorMsgs_1 = require("../_constantes/ErrorMsgs");
const InfoMsgs_1 = require("../_constantes/InfoMsgs");
const NamingConvention_1 = require("../_constantes/NamingConvention");
class Initializer {
    constructor(successMessage, mkdirConfigDir = false) {
        this.successMessage = successMessage;
        this.mkdirConfigDir = mkdirConfigDir;
    }
    static _initTemplate(tempDirPath) {
        const configFileResult = TemplateReader_1.TemplateReader.getTemplateConfigFile(tempDirPath);
        if (configFileResult instanceof Error)
            return configFileResult;
        const checkingResult = TemplateReader_1.TemplateReader.checkConfigFileProperties(configFileResult);
        if (checkingResult instanceof Error)
            return checkingResult;
        TemplateRepository_1.TemplateRepo.saveTemplate(configFileResult.template, tempDirPath);
        return configFileResult.template;
    }
    static initExempleTemplate() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const configDirPath = path_1.default.resolve(config_1.nameConfigDir);
                const exampleTemplate = {
                    dirName: `example-${config_1.edging}${NamingConvention_1.NamingConvention.paramCase}${config_1.edging}-repository`,
                    files: [
                        { fileName: `implementation-${config_1.edging}${NamingConvention_1.NamingConvention.paramCase}${config_1.edging}-repository.ts`, data: "" },
                        { fileName: `${config_1.edging}${NamingConvention_1.NamingConvention.paramCase}${config_1.edging}-repository.test.ts`, data: "" },
                        { fileName: `${config_1.nameConfigFile}`, data: `{
  "template" : "repository",
  "componentWorkDir" :"src/core"
}`
                        },
                        { fileName: `interface-${config_1.edging}${NamingConvention_1.NamingConvention.paramCase}${config_1.edging}-repository.ts`, data: `interface I${config_1.edging}${NamingConvention_1.NamingConvention.pascalCase}${config_1.edging}Repo {        
  anyProperty : string        
  hello : number        
  getAll${config_1.edging}${NamingConvention_1.NamingConvention.pascalCase}${config_1.edging} () : Object[]        
  getOne${config_1.edging}${NamingConvention_1.NamingConvention.pascalCase}${config_1.edging} () : Object        
}`
                        },
                    ]
                };
                yield (0, FileHandler_1.fileHandler)(configDirPath, exampleTemplate.dirName, exampleTemplate.files);
                console.log(InfoMsgs_1.InfoMsgs.SUCCESS_EXAMPLE_TEMPLATE_CREATON);
            }
            catch (err) {
                console.log(ErrorMsgs_1.ErrorMsgs.UNEXPECTRED_EXAMPLE_TEMPLATE_CREATION_ERR);
            }
        });
    }
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const configDirPath = path_1.default.resolve(config_1.nameConfigDir);
                const initConfigDirResult = yield this.initConfigDir(configDirPath);
                if (initConfigDirResult instanceof Error)
                    return initConfigDirResult;
                const configDirData = yield (0, promises_1.readdir)(configDirPath, { withFileTypes: true });
                const templates = configDirData
                    .filter(dirent => dirent.isDirectory())
                    .map(temp => ({ dirName: temp.name, dirPath: `${configDirPath}/${temp.name}` }));
                TemplateRepository_1.TemplateRepo.clear();
                const initTempaltesResult = this._initTemplates(templates);
                if (initTempaltesResult instanceof Error)
                    return initTempaltesResult;
                console.log(this.successMessage);
            }
            catch (err) {
                console.log(ErrorMsgs_1.ErrorMsgs.UNEXPECTED_ERR("l'initialisation", err));
            }
        });
    }
    _initTemplates(templates) {
        const errorsMessages = [];
        const dupliErrors = {};
        const initializedTemplates = new Map();
        templates.forEach(tempDirInfo => {
            const currTempDirPath = tempDirInfo.dirPath;
            const initTempResult = Initializer._initTemplate(currTempDirPath);
            if (initTempResult instanceof Error)
                return errorsMessages.push(initTempResult.message);
            if (initializedTemplates.has(initTempResult)) {
                if (!(initTempResult in dupliErrors))
                    dupliErrors[initTempResult] = new Set();
                const firstInitializedPath = initializedTemplates.get(initTempResult);
                dupliErrors[initTempResult].add(firstInitializedPath);
                dupliErrors[initTempResult].add(currTempDirPath);
            }
            initializedTemplates.set(initTempResult, currTempDirPath);
        });
        if (Object.keys(dupliErrors).length)
            for (const tempName in dupliErrors) {
                const paths = [...dupliErrors[tempName].values()];
                errorsMessages.push(ErrorMsgs_1.ErrorMsgs.DUPLICATE_TEMPLATE_NAME(tempName, paths));
            }
        if (errorsMessages.length)
            return new Error(`\n${(0, config_1.cWarning)("Des erreurs se sont produites pendant l'initialisation des templates:")}\n\n-${errorsMessages.join("\n-")}`);
    }
    initConfigDir(mkdirPath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (fs_1.default.existsSync(mkdirPath))
                return true;
            else if (this.mkdirConfigDir) {
                fs_1.default.mkdirSync(mkdirPath);
                yield Initializer.initExempleTemplate();
                return true;
            }
            return new Error(ErrorMsgs_1.ErrorMsgs.CONFIG_DIR_MISS);
        });
    }
}
exports.Initializer = Initializer;
//# sourceMappingURL=Initializer.js.map